/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt Mobility Components.
**
** $QT_BEGIN_LICENSE:LGPL$
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the Technology Preview License Agreement accompanying
** this package.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
**
**
**
**
**
**
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef SIMULATORCOMMON_H
#define SIMULATORCOMMON_H

#include <qsensorbackend.h>
#include "qsensordata_simulator_p.h"

QT_BEGIN_HEADER

class QTimer;

QTM_BEGIN_NAMESPACE

namespace Simulator
{
    class MobilityConnection;

    class SensorsConnection : public QObject
    {
        Q_OBJECT
    public:
        SensorsConnection(MobilityConnection *mobilityCon);
        virtual ~SensorsConnection() {}

        void getInitialData();

    private slots:
        void setAmbientLightData(const QtMobility::QAmbientLightReadingData &);
        void setAccelerometerData(const QtMobility::QAccelerometerReadingData &);
        void setMagnetometerData(const QtMobility::QMagnetometerReadingData &);
        void setCompassData(const QtMobility::QCompassReadingData &);
        void setProximityData(const QtMobility::QProximityReadingData &);
        void initialSensorsDataSent();

    private:
        MobilityConnection *mConnection;
        bool mInitialDataReceived;
    };
} // end namespace Simulator

class SimulatorCommon : public QSensorBackend
{
public:
    SimulatorCommon(QSensor *sensor);

    void start();
    void stop();
    virtual void poll() = 0;
    void timerEvent(QTimerEvent * /*event*/);

private:
    int m_timerid;
};

QAccelerometerReadingData get_qtAccelerometerData();
QMagnetometerReadingData get_qtMagnetometerData();
QAmbientLightReadingData get_qtAmbientLightData();
QCompassReadingData get_qtCompassData();
QProximityReadingData get_qtProximityData();

QTM_END_NAMESPACE
QT_END_HEADER

#endif

